!=============================================================================================================
! ControlMonitor Class          ( ControlMonitor.inc )
! Version 2.4
! Last Updated: 04-12-2004
!
!=============================================================================================================
! This template/class set was assembled by Charles Edmonds and is based on code and
! much help from Arnor Baldvinsson.  It also contains suggestions from other members
! of the Clarion Development Community.
!
! Thanks to all who contributed!
!=============================================================================================================
! Background
!
! This class / template set was written to provide a better way to execute code when the
! the contents of ANY field or control was changed - either by direct user interaction or
! by a direct code assignment (such as Variable=Value).  The goal was to eliminate the need
! to call a routine or enter code at each embed point on a screen with many controls.
!
! The idea for the set (and much of the initial code) came from Arnor Baldvinsson in response
! to an inquiry in the Clarion Newsgroups by Charles Edmonds.  Charles was responsible for
! coding the actual class structure and then implementing the template to make it easy to use.
!
! Support was added for the Sysinternals DebugView program so that a programmer could view output
! as the class was working without relying on STOP, HALT or MESSAGE statements (which are known
! to interfer with the code processing.
!---------------------------------------------------------------------------------------------
! Class Dynamics
!
! The class works by building a queue of the FEQ numbers and a CRC for the contents of each control 
! as the window is initialized.  Then when a Field Event is generated or an EVENT:Accepted is posted,
! the class checks the CRC of all controls against the data in the queue.  If the CRC's do not match
! then any code the programmer has placed in a ROUTINE that is generated by the template is called.
!
! In the case of a Field Event (such as a entry, spin control, combo, etc.) - in other words a
! direct action as the result of the user interacting with the program, the "HasChanged" method
! is called and the contents of the specific field are compared against the record in the queue.
! IF they do not match, the queue is updated with the new CRC and the code in the routine is called.
!
! In the case of an EVENT:Accepted, the "AnyChanged" method of the class is called and the contents
! of each control is compared against the value stored in the queue.  If they do not match, then the
! queue record is updated with the new CRC and the code in the routine is called.
!
!
! This dual approach method makes sure that the code in the routine is ALWAYS called if a change in
! the data occurs - whether by user interaction with the program, or by direct code assignment.
!---------------------------------------------------------------------------------------------
! Revisions
! 1.1   Changed class to trap field events and Event:Accepted
! 1.2   Added support for SysInternals Debugger
! 1.3   Changed to CRC method of verifying data
! 1.4   Localized instances of the class
! 1.5   Updated template to allow programmers to name class instances
! 2.0   Added Extended CASE statement generation and support for multi-DLL applications
! 2.1   Changed name of embed points in Extended CASE for clarity
! 2.2   Added a CLEAR( %classname.lastchanged ) after the CASE statement to avoid duplicate code calls
! 2.3   Added a checkbox on the Global Extension tab to disable ALL output to DebugView
!       (This makes it easy to be sure all output is turned OFF when you ship your program)
! 2.4   Added support for LIST controls in Extended CASE template
!---------------------------------------------------------------------------------------------
! Instructions
!
! 1.    Add the ControlMonitor Global Extension Template to the Global section of your program.
! 2.    Add the ControlMonitor Detect Template to each procedure where you want to use it.
! 3.    Set the name for the localized instance of the class (or accept the default value)
! 4.    If you want to output to the Sysinternals DebugView program, check the box on the extension template.
!
!       You can download the debugger from www.sysinternals.com if you do not have it.
!       Run the DBGVIEW.EXE executable before you start your program to see the output.
!       When you no longer need it, just uncheck the box on the template extension tab.
!
!       Note:  You can also set the output to the debugger ON/OFF via code by setting the following
!              value in your code AFTER the INIT function has been called.
!
!              classinstancename.wDebugAll = True  (or False)
!---------------------------------------------------------------------------------------------
! Changes / Additions
!
! If you make useful changes to this template or class set, please email a copy to Charles Edmonds
! at "ControlMonitor@lansrad.com" so we can update the copies at the download centers
!---------------------------------------------------------------------------------------------

!ABCIncludeFile
 OMIT('_EndOfInclude_',_ControlMonitorPresent_)
_ControlMonitorPresent_     EQUATE(1)


!------------------------------------------------------------
!New Class Declarations
!------------------------------------------------------------
ControlMonitor  CLASS, TYPE,MODULE('ControlMonitor.clw'),LINK('ControlMonitor.clw',_ABCLinkMode_),DLL(_ABCDllMode_)
Initialized         BYTE
LastChanged         LONG    !FEQ of last control to change.  Added 8-11-03, Charles Edmonds
Controls            &CQ
Init                Procedure
Kill                Procedure
IsSelected          Procedure(Long pFEQ),Byte
HasChanged          Procedure(Long pFEQ),Byte
AnyChanged          Procedure,Byte
wDebug              Procedure(String pS)
wDebugAll           BYTE
GetCRC              Procedure(Long pFEQ),Long
RemoveControl       Procedure(Long pFEQ)
RegisterControl     Procedure(Long pFEQ)
                 END !ControlMonitor




_EndOfInclude_


