#!=============================================================================================================
#Template(ControlMonitor,'Template to Detect Changes on ANY control'),Family('cw20'),Family('ABC')
#!=============================================================================================================
#!
#! ControlMonitor Class
#! Version 2.4
#! Last Updated: 04-12-2004
#!
#!=============================================================================================================
#! This template/class set was assembled by Charles Edmonds and is based on code and
#! much help from Arnor Baldvinsson.  It also contains suggestions from other members
#! of the Clarion Development Community.
#!
#! Thanks to all who contributed!
#!=============================================================================================================
#!
#!
#!=============================================================================================================
#! Revision: 2.2
#!
#! Note to 2.0 BETA testers.  Version 2.1 changed the name of the embeds point when using the extended CASE option.
#!                        This was for clarity.  If you have code in the old points, you'll need to move it.
#!                        Version 2.2 also adds a CLEAR( classname.lastchanged ) after the CASE statement to avoid
#!                        duplication of code execution when using the Extended CASE statement.
#!
#!
#! This revision is the result of some suggestions made by Alain Doudiet and some redesign by Charles Edmonds.
#!
#! A new option has been added to the extension template that will build an Extended CASE statement of all the
#! controls on the window.  This allows you to place control-specific code that will be executed whenever the
#! contents of that specific control have changed.
#!
#! Why have this option?
#!
#! The advantage of doing this over using the standard Clarion embed points is that you KNOW that the code will
#! be executed if the data changes - AND - all your embedded change code (even field specific code) is located
#! in the ONE case statement.
#!
#! If you use this option, then THREE new embed points will be generated:
#!
#! BEFORE-CASE:   Statements to be executed BEFORE the generated CASE statement.  Code inserted here will be executed
#!                whenever ANY data has changed.  This is exactly the way the original ControlMonitor code worked.
#!
#! CASE:          This will be a generated CASE statement containing an 'OF *CONTROL_TO_MONITOR*' for each of the
#!                controls on the window.  This allows you to insert control specific code that will be executed
#!                whenever the value of that specific control has changed.
#!
#!                NOTE:  Unlike using a CASE FIELD() statement (which is only executed if the user actually changed
#!                       data with the control itself), the CASE statement generated by this template will execute
#!                       whenever the data is changed by any means (even another direct program statement).
#!
#!
#!                IMPORTANT:  The CASE statement is generated by the template.  All you do is insert your code.
#!
#!
#! AFTER-CASE:    Statements to be executed AFTER the generated CASE statement.  Code inserted here will be executed
#!                whenever ANY data has changed.  This point can be used to execute code statements if needed AFTER
#!                code executed in either Point 1 or Point 2 had changed something.
#!
#! If you do not use the Extended CASE option, then a single Control Monitor code will be generated just like
#! in the original version.
#!
#! UPGRADING:  If you enable the Extended CASE option and are upgrading from an older version of this template,
#!             then your existing code will be placed in the BEFORE-CASE embed point.
#!
#!
#!
#! This Revision also supports Multi-DLL applications (courtesy of Alain Doudiet)
#!
#! ** Special Thanks go to Alain Doudiet, PSD Srl for his contributions and suggestions!
#!
#!=============================================================================================================
#! Revision: 2.3  Added a checkbox on the Global Extension tab to disable ALL output to DebugView
#!                * This makes it easy to be sure all output is turned OFF when you ship your program
#!                * Note that procedure extension checkboxes will show as disabled when the global is checked
#!=============================================================================================================
#! Revision: 2.4  Added support for LIST controls in Extended CASE template
#!=============================================================================================================
#Extension(ControlMonitor,'Monitor changes on ANY control'),Application
#!=============================================================================================================

#SHEET
  #TAB('General')
    #DISPLAY
    #DISPLAY('ControlMonitor Class')
    #DISPLAY('Version 2.4')
    #DISPLAY('Last Updated: 04-12-2004')
    #DISPLAY
    #DISPLAY('Global Extension Template')
    #DISPLAY
    #DISPLAY('Adds global support for the ControlMonitor class')
    #DISPLAY('to this application.')
    #DISPLAY
    #PROMPT( 'Disable ALL Output to SysInternals DebugView',CHECK),%DisableSysInternalsFlag, AT(10)
    #DISPLAY('(Useful when distributing a program)')
    #DISPLAY
    #DISPLAY
    #PROMPT ('Control Monitor class (defined in another DLL)',Check),%MonitorExternal,at(10)
    #DISPLAY('      Select this option for Multi-DLL use and this is')
    #DISPLAY('      NOT the DLL where the class was defined.')
    #DISPLAY
    #DISPLAY('Credits:')
    #DISPLAY('This template/class set was assembled by')
    #DISPLAY('Charles Edmonds and is based on code and')
    #DISPLAY('much help from Arnor Baldvinsson.  It also')
    #DISPLAY('contains suggestions from other members')
    #DISPLAY('of the Clarion Development Community.')
    #DISPLAY('Thanks to all who contributed!')
    #DISPLAY
    #DISPLAY('This template/class set is provided as a FREE tool.')
    #DISPLAY
    #DISPLAY('DISCLAIMER:')
    #DISPLAY('The authors and contributors to this template')
    #DISPLAY('set assume NO Liability and take no responsibility')
    #DISPLAY('for its operation or how it affects your program.')
    #DISPLAY('Use at your own risk!')
  #ENDTAB
  #TAB('Description')
    #DISPLAY
    #DISPLAY('The ControlMonitor class monitors the data stored')
    #DISPLAY('in the controls of a procedure window and executes')
    #DISPLAY('code defined by the programmer whenever the data')
    #DISPLAY('has been modified.')
    #DISPLAY
    #DISPLAY('The class monitors changes to an individual control')
    #DISPLAY('through the TakeFieldEvent and also through direct')
    #DISPLAY('data assignments such as a Variable=Value statement')
    #DISPLAY('in the EVENT:Accepted embed of a control.')
    #DISPLAY
    #DISPLAY('This class is especially useful when there is an update')
    #DISPLAY('routine that needs to be called each time a setting is')
    #DISPLAY('changed on a window with a large number of controls.')
    #DISPLAY('It eliminates the need to place a call to the update')
    #DISPLAY('code into the embeds of each control.')
    #DISPLAY
    #DISPLAY('The class works by building a queue of FEQ numbers')
    #DISPLAY('for each control and then by maintaining a CRC')
    #DISPLAY('for the data stored in each control.  When the class')
    #DISPLAY('is called, the CRC is compared to that of the new data')
    #DISPLAY('and the update code is called if the data has changed.')
  #ENDTAB
  #TAB('Updates and Suggestions')
    #DISPLAY
    #DISPLAY('This Free template set will be posted at the Clarion')
    #DISPLAY('download centers and in the SoftVelocity newsgroups.')
    #DISPLAY
    #DISPLAY
    #DISPLAY('CONTRIBUTE:')
    #DISPLAY('This template/class set is a work in progress.')
    #DISPLAY
    #DISPLAY('If you have suggestions or make useful improvments')
    #DISPLAY('to the template or class, please send them to:')
    #DISPLAY
    #DISPLAY('Charles Edmonds at: ControlMonitor@lansrad.com')
    #DISPLAY
    #DISPLAY('Be sure and check out the fine Clarion development tools')
    #DISPLAY('by Arnor Baldvinsson at www.icetips.com')
    #DISPLAY('- this is a shameless plug in THANKS for all his help -')
  #ENDTAB
#ENDSHEET

#AT(%AfterGlobalIncludes)
   !Include Changed Class
   INCLUDE( 'ControlMonitor.inc' )
#ENDAT

#AT(%DLLExportList),WHERE(NOT %MonitorExternal AND %ProgramExtension='DLL')
  INIT@F14CONTROLMONITOR             @?
  KILL@F14CONTROLMONITOR             @?
  ISSELECTED@F14CONTROLMONITORl      @?
  HASCHANGED@F14CONTROLMONITORl      @?
  ANYCHANGED@F14CONTROLMONITOR       @?
  TYPE$CONTROLMONITOR                @?
  VMT$CONTROLMONITOR                 @?
#ENDAT


#!=============================================================================================================
#Extension(ControlMonitorDetect,'Detect Changes on ANY control in this procedure'),PROCEDURE, REQ(ControlMonitor)
#!=============================================================================================================
#SHEET
  #TAB('General')
    #DISPLAY
    #DISPLAY('ControlMonitor Class')
    #DISPLAY('Version 2.4')
    #DISPLAY('Last Updated: 04-12-2004')
    #DISPLAY
    #DISPLAY('Procedure Extension Template')
    #DISPLAY
    #DISPLAY('Adds local support for the ControlMonitor class')
    #DISPLAY('to this procedure.')
    #DISPLAY
    #PROMPT('Class Name:',@s50),%CEClassName,Default(%PROCEDURE)
    #DISPLAY
    #DISPLAY('Note:')
    #DISPLAY('The class name defaults to the procedure name')
    #DISPLAY('where it is called from so it is easy to tell')
    #DISPLAY('which procedure is generating the output.')
    #DISPLAY
    #PROMPT('Use Extended CASE routine',CHECK),%ExtendedRoutine, AT(10)
    #DISPLAY
    #DISPLAY('This routine builds a CASE statement of the')
    #DISPLAY('controls automatically so you can insert')
    #DISPLAY('specific code to be executed when the data')
    #DISPLAY('in one or more controls has changed.')
    #DISPLAY
    #DISPLAY('It also has BEFORE-CASE and AFTER-CASE embed')
    #DISPLAY('points where you can insert code that will')
    #DISPLAY('always be executed when data has changed.')
    #DISPLAY
    #DISPLAY('Credit for the Extended CASE routine idea')
    #DISPLAY('goes to Alain Doudiet, PSD Srl')
    #DISPLAY
  #ENDTAB
  #TAB('SysInternals DebugView Support')
    #DISPLAY
    #DISPLAY('This template has built-in support for the free')
    #DISPLAY('debug tools by SysInternals.')
    #DISPLAY
    #DISPLAY('You can download them at www.sysinternals.com')
    #DISPLAY
    #ENABLE( %DisableSysInternalsFlag = 0 )
    #PROMPT('Output to SysInternals DebugView',CHECK),%OutputToSysInternalsFlag, AT(10)
    #ENDENABLE
    #DISPLAY
    #DISPLAY('If this checkbox is disabled, then the output')
    #DISPLAY('has been turned OFF on the global extension tab')
    #DISPLAY
    #DISPLAY
    #DISPLAY('Note:')
    #DISPLAY('Run the DBGVIEW.EXE executable to view the output.')
  #ENDTAB
 #ENDSHEET


#AT(%DataSection),PRIORITY(5000)
!Declare instance of ChangedClass
%CEClassName    ControlMonitor
#ENDAT


#AT(%WindowManagerMethodCodeSection,'Init','(),BYTE'),PRIORITY( 9001 )
%CEClassName.Init               !Initialize ControlMonitor
#IF( %OutputToSysInternalsFlag = 1 AND %DisableSysInternalsFlag = 0)
%CEClassName.wDebugAll = True   !Enable output to SysInternals Debugger ( Get it from www.sysinternals.com )
#ELSE
%CEClassName.wDebugAll = False   !Disable output to SysInternals Debugger ( Get it from www.sysinternals.com )
#ENDIF
#ENDAT

#AT(%WindowManagerMethodCodeSection,'Kill','(),BYTE'),PRIORITY( 5001 )
%CEClassName.Kill           !Kill ControlMonitor
#ENDAT

#AT(%WindowManagerMethodCodeSection,'TakeEvent','(),BYTE'),PRIORITY( 5000 )
  !Process ControlMonitor Code
  If Event() = Event:Accepted
    DISPLAY
    If %CEClassName.AnyChanged()
      !Process user code in the routine
      DO ControlMonitorCode
    End
  End
#ENDAt

#AT(%WindowManagerMethodCodeSection,'TakeFieldEvent','(),BYTE'),PRIORITY( 5001 )
!Insert embed point for ControlMonitor
If %CEClassName.IsSelected(Field())
  If %CEClassName.HasChanged(Field())
    !Process user code in the routine
    DO ControlMonitorCode
  End
End
#ENDAT


#AT(%ProcedureRoutines),PRIORITY( 4000 )
ControlMonitorCode        ROUTINE
#IF(%ExtendedRoutine)
  #EMBED(%ControlMonitorBeforeCASE,'ControlMonitor: Insert BEFORE-CASE Code HERE to Process when changes are made on ANY CONTROL in this procedure'),LABEL

    CASE %CEClassName.LastChanged
#FOR(%Control),WHERE(%Control)
#IF(%ControlType='ENTRY' OR %ControlType='SPIN'OR %ControlType='COMBO'OR %ControlType='TEXT' OR %ControlType='CHECK' OR %ControlType='OPTION' OR %ControlType='LIST')
    OF %Control
  #EMBED(%ControlAccepted,'ControlMonitor: Insert CASE Code HERE to Process when changes are made on a SPECIFIC CONTROL in this procedure'),%Control
#ENDIF
#ENDFOR
  END
  #EMBED(%ControlMonitorAfterCASE,'ControlMonitor: Insert AFTER-CASE Code HERE to Process when changes are made on ANY CONTROL in this procedure'),LABEL
  !Clear %CEClassName.LastChanged after the case statement
  CLEAR(%CEClassName.LastChanged)
#ELSE
  #EMBED(%ControlMonitorAfterChanges,'ControlMonitor: Insert Code HERE to Process when changes are made on ANY CONTROL in this procedure'),LABEL
#END
#ENDAT




